using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Microsoft.TeamFoundation.Client;
using Microsoft.TeamFoundation.WorkItemTracking.Client;
using Microsoft.TeamFoundation.Server;

namespace AreaImportExportTool
{
    public partial class MainForm : Form
    {
        public MainForm()
        {
            InitializeComponent();

            chkIterationStructure_CheckedChanged(this, EventArgs.Empty);

            this.txtServername.Text = System.Net.Dns.GetHostName().ToUpper();
        }

        private void chkIterationStructure_CheckedChanged(object sender, EventArgs e)
        {
            cmdExport.Enabled = cmdImport.Enabled =
                (chkAreaStructure.Checked || chkIterationStructure.Checked) &&
                !(String.IsNullOrEmpty(txtServername.Text) ||
                txtServername.Text.Trim().Length == 0) &&
                !(String.IsNullOrEmpty(txtTeamProject.Text) ||
                txtTeamProject.Text.Trim().Length == 0);
        }

        private void cmdExport_Click(object sender, EventArgs e)
        {
            string servername = txtServername.Text;
            string teamProject = txtTeamProject.Text;

            SaveFileDialog dlg = new SaveFileDialog();
            dlg.CheckPathExists = true;
            dlg.Title = "Export Area/Iteration structure";
            dlg.DefaultExt = ".nodes";
            dlg.Filter = "TFS Node Structure (*.nodes)|*.nodes";
            if (dlg.ShowDialog(this) == DialogResult.OK)
            {
                string filename = dlg.FileName;

                using (TeamFoundationServer tfs = TeamFoundationServerFactory.GetServer(servername, new UICredentialsProvider()))
                using (WaitState waitState = new WaitState(this))
                {
                    tfs.EnsureAuthenticated();
                    WorkItemStore store = (WorkItemStore)tfs.GetService(typeof(WorkItemStore));
                    Project proj = store.Projects[teamProject];

                    using (FileStream fs = new FileStream(filename, FileMode.Create, FileAccess.Write))
                    using (StreamWriter writer = new StreamWriter(fs))
                    {
                        writer.WriteLine("NODES");
                        writer.WriteLine(String.Format("{0}, Version {1}", Application.ProductName, Application.ProductVersion));
                        writer.WriteLine("Copyright (C) 2007 " + Application.CompanyName);

                        if (chkAreaStructure.Checked)
                        {
                            WriteNodes(proj.AreaRootNodes, writer, "A");
                        }

                        if (chkIterationStructure.Checked)
                        {
                            WriteNodes(proj.IterationRootNodes, writer, "I");
                        }

                        writer.Close();
                    }
                }

                MessageBox.Show("Export successful.");
            }
        }

        private static void WriteNodes(NodeCollection nodes, StreamWriter writer, string prefix)
        {
            foreach (Node node in nodes)
            {
                writer.Write(prefix);
                writer.WriteLine(node.Path.Substring(node.Path.IndexOf(@"\")));

                if (node.ChildNodes.Count > 0)
                {
                    WriteNodes(node.ChildNodes, writer, prefix);
                }
            }
        }

        private void cmdImport_Click(object sender, EventArgs e)
        {
            string servername = txtServername.Text;
            string teamProject = txtTeamProject.Text;

            OpenFileDialog dlg = new OpenFileDialog();
            dlg.CheckFileExists = true;
            dlg.Title = "Import Area/Iteration structure";
            dlg.DefaultExt = ".nodes";
            dlg.Filter = "TFS Node Structure (*.nodes)|*.nodes";
            if (dlg.ShowDialog(this) == DialogResult.OK)
            {
                string filename = dlg.FileName;

                using (TeamFoundationServer tfs = TeamFoundationServerFactory.GetServer(servername, new UICredentialsProvider()))
                using (WaitState waitState = new WaitState(this))
                {
                    tfs.EnsureAuthenticated();

                    WorkItemStore store = (WorkItemStore)tfs.GetService(typeof(WorkItemStore));
                    Project proj = store.Projects[teamProject];

                    NodeInfo rootAreaNode = null;
                    NodeInfo rootIterationNode = null;

                    ICommonStructureService css = (ICommonStructureService)tfs.GetService(typeof(ICommonStructureService));
                    foreach (NodeInfo info in css.ListStructures(proj.Uri.ToString()))
                    {
                        if (info.StructureType == "ProjectModelHierarchy")
                        {
                            rootAreaNode = info;
                        }
                        else if (info.StructureType == "ProjectLifecycle")
                        {
                            rootIterationNode = info;
                        }
                    }

                    using (FileStream fs = new FileStream(filename, FileMode.Open, FileAccess.Read))
                    using (StreamReader reader = new StreamReader(fs))
                    {
                        string nextLine = reader.ReadLine();
                        if (nextLine != "NODES")
                        {
                            MessageBox.Show("Wrong file format!");
                            return;
                        }

                        reader.ReadLine();
                        reader.ReadLine();

                        while (!reader.EndOfStream)
                        {
                            nextLine = reader.ReadLine();
                            if (nextLine.StartsWith("A") && chkAreaStructure.Checked)
                            {
                                CreateNode(css, nextLine.Substring(2), rootAreaNode);
                            }
                            else if (nextLine.StartsWith("I") && chkIterationStructure.Checked)
                            {
                                CreateNode(css, nextLine.Substring(2), rootIterationNode);
                            }
                            else
                            { // Ignore other lines
                            }
                        }
                        reader.Close();
                    }

                    MessageBox.Show("Import successful.");
                }
            }
        }

        private void CreateNode(ICommonStructureService css, string nodeName, NodeInfo rootNode)
        {
            if (!nodeName.Contains("\\"))
            {
                css.CreateNode(nodeName, rootNode.Uri);
            }
            else
            {
                int lastBackslash = nodeName.LastIndexOf("\\");
                NodeInfo info = css.GetNodeFromPath(rootNode.Path + "\\" + nodeName.Substring(0, lastBackslash));
                css.CreateNode(nodeName.Substring(lastBackslash + 1), info.Uri);
            }
        }

        private void cmdClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private class WaitState : IDisposable
        {
            private Form parent;

            public WaitState(Form parent)
            {
                parent.Cursor = Cursors.WaitCursor;
                parent.Enabled = false;
                this.parent = parent;
            }

            void IDisposable.Dispose()
            {
                parent.Cursor = Cursors.Default;
                parent.Enabled = true;
            }
        }
    }
}